#' Identify outliers and replace outliers and NAs in biomarkers with column average
#' @details  It may be desirable to identify outliers within the dataset and to fill NAs for biomarkers in preparation for Principal Components Analysis. This function will: 1) identify biomarker samples that are greater than 4 standard deviations from the group mean, 2) replace these outliers with the group mean if desired, and 3) replace NAs with the group mean if desired. Samples from multiple chips can be combined for use with this function and this is desired when samples from a single population are on multiple chips. 
#' @param normalizedchip the output from normalize (or a combination of multiple normalized outputs), although this could also be a non-normalized chip
#' @param NA.action 'none' will do nothing, 'replace' will replace NAs with column averages
#' @param outlier.action 'NULL' will do nothing, 'replace' will replace outliers with column averages, 'find' will identify and report outliers
#' @return data the altered (if actions are not set to 'none')
#' @return outl a dataframe containing the row number, the assay name, the SD x 4, and the outlier value
#' @return NAs  a dataframe containing the assay and sample combos of NAs that were replaced with averages
#' @export 

NAs.outliers<-function(normalizedchip,NA.action="replace",outlier.action=NULL){
  
  chip<-normalizedchip
  
  bcol<-NULL
  misc<-NULL
  samps<-NULL
  
  for(i in 1:nrow(chip)){
    if(substr(chip[i,1],1,4)!="cont"){samps<-append(samps,i)}          # identifies any rows that are not controls and collects them
  }
  
  chip<-chip[samps,]                                                         
  
  for(i in 1:ncol(chip)){  
    if(substr(colnames(chip)[i],1,1)=="b"){bcol<-append(bcol,i)}       #identify columns of biomarker genes
    else{misc<-append(misc,i)}}
  
  
  
  #    library(outliers)
  
  outliers<-NULL
  complete<-NULL
  reped<-NULL
  
  for(i in bcol){                                            # for each biomarker column
    if(!is.null(outlier.action)){
      #med<-median(chip[,i])
      #MAD<-median(abs(chip[,i]-med))
      
      #err<-sd(na.omit(chip[,i]))*4                             # calculates std. dev. *4
      #l<-mean(na.omit(chip[,i]))-err                           # gives you a low SD, which is kinda bullshit since its negative
      #h<-mean(na.omit(chip[,i]))+err                           # gives you a high SD
      qs<-quantile(chip[,i],c(0.01,0.99),na.rm=TRUE)
      
      low<-which(chip[,i]<as.numeric(qs[1]))                                   # identifies samples that are outside SD*4
      high<-which(chip[,i]>as.numeric(qs[2]))
      out<-append(low,high)
      outl<-cbind(out,chip[out,1],colnames(chip[i]),round(chip[out,i],3),chip$RNAconc[out])  # combines the row #, biomarker name, outlier value, and RNAconc
      
      
      if(length(out)>0){outliers<-rbind(outliers,outl)}        # adds it to the total list
      
      if(length(which(!is.na(chip[,i])))>0 & length(out)>0){   # need to specify that we are only interested in columns that have data and outliers
        if(outlier.action=="replace"){                           # if "replace" is specified...
          m<-mean(na.omit(chip[-out,i]))                         # calculate column mean without the outlier
          chip[out,i]<-m                                         # replace the outlier with column mean
        }}}
    if(length(which(is.nan(chip[,i])))>0){chip[which(is.nan(chip[,i])),i]<-NA}
    
    if(length(which(is.na(chip[,i])))>0){
      if(NA.action=="replace"){                                
        nonnas<-which(!is.na(chip[,i]))                       # collect true values (not NAs)
        if(length(nonnas)>0){                                   
          m<-mean(chip[nonnas,i])                             # calculate mean for column
          rep<-which(is.na(chip[,i]))                          # find NAs to replace
          for(j in rep){
            reped<-rbind(reped,cbind(j,chip[j,1],colnames(chip)[i]))}
          chip[rep,i]<-m}                                   # replace NAs with mean
      }}}  
  
  if(length(reped[,1])>0){colnames(reped)<-c("Rownum","Sample","Assay")        # give the NAs list some colnames
                          reped<-reped[order(reped[,2]),]}                                       # sort the list by sample     
  
  if(length(outliers[,1])>0){colnames(outliers)<-c("Rownum","Sample","Assay","Value","RNAconc")  # give the outlier file some column names
                             outliers<-outliers[order(outliers[,2],outliers[,3]),]}                       # sort the outlier dataset
  
  if(!is.null(outlier.action)){
    print("The following were found to be outliers at the extreme ends of the distribution.",quote=FALSE)
    print(outliers)}
  if(NA.action=="replace"){print("The following samples had NAs for the listed Assays and were replaced with the mean for that assay:",quote=FALSE)
                           print(reped)
  }
  
  list(data = chip, outl = outliers, NAs = reped)  # the cleaned up chip is found in $data, the list of outliers is in $outl
  
}
