#' Clean and average duplicate heatmap data
#' @param   importedchip      this is the output object from the importchip() function
#' @param   LOD               applies LOD screening, an integer applies a global Ct valye, and "assay_LOD" applies assay specific values, default = NULL 
#' @return  data              the cleaned dataset for analysis
#' @return  fails             a list of the sample/assay combos that failed and were removed
#' @return  vicpos            a list of VIC positive microbe samples   
#' @return  single            a list of microbe samples that were not positive in both duplicates
#' @return  blankpos          a list of blanks that were positive
#' @return  effdata           a dataset to be used in the effcalc() function
#' @return  overLOD           a list of samples that had Cts above the LOD, if specified
#' @return  LODlist           a list of the LOD limits that were applied (CSAS Table 3)
#' @details This function will, in the following order: 1) apply pass/fail screening to FAM data, 2) apply VIC screening to FAM data, 3) replace high Cps for microbes 
#' with NA if LOD is specified, 4) average all duplicate assays, 5) identify positives in blanks,. If a microbe assay is missing a duplicate, it is assigned an NA. If a 
#' biomarker is missing a duplicate, it is replaced with the value of the existing duplicate (no averaging occurs). 
#' 
#' The LOD values, which come from the 2015 CSAS (Table 3), are created within the function and therefore can not be edited by end users. I can change this if there is a desire for it! 
#' @export


cleanavg<-function(importedchip, LOD=NULL){
  
f<-importedchip
  
trim.leading <- function (x)  sub("^\\s+", "", x)               # introduce a function to remove blanks from assay names

  #### PASS/FAIL SCREENING FOR FAM AND vIC #######
  fails<-NULL
  for(i in 1:nrow(f)){            # replace all fails and 999s with NA
    if(f[i,12]=="Fail" & f[i,9]<999){
      fails<-rbind(fails,f[i,c(1,2,5,9)])}   # this records what assays failed with values less than 999
    if(f[i,12]=="Fail"){f[i,9]<-NA}      # this assigns NA to failed FAM assays
    if(f[i,17]=="Fail"){f[i,14]<-NA}     # this assigns NA to failed VIC assays
  }
  x<-match(999,f[,9])
  if(!is.na(x)){f[x,9]<-NA}         # failsafe: any leftover 999s will be turned to NA
  #### VIC SCREENING #######  QUESTION!!! SHOULD LOD BE APPLIED TO VIC?! Answer as of Jan 2016 = NO. VIC screening should be as conservative as possible
  
  vicpos<-NULL
  for(i in 1:nrow(f)){
    if(f[i,3]!="Standard" & substr(f[i,5],1,1)=="m" & !is.na(f[i,14])==TRUE){  # find data that are not standards and are VIC pos
      vicpos<-rbind(vicpos,f[i,c(1,2,5,9)])            # record these data
      f[i,9]<-NA                                     # change FAM to NA
    }
  }
  
  #### AVERAGE DUPLICATES, DROP SINGLETONS ####
  rn<-nrow(f)
  g<-data.frame("ID" = character(rn), "Sample" = character(rn), 
                "Stype" = character(rn),"rConc" = numeric(rn), 
                "FAssay" = character(rn), "FamCt" = numeric(rn),
                stringsAsFactors=FALSE)       # create a dataframe with the proper classes to hold the results
  
  singletons<-NULL
  for(i in 1:rn){
    g[i,]<-f[i,c(1:5,9)]                        # move data over to new dataframe
    rep<-which(f[,2]==g[i,2] & f[,5]==g[i,5])    # find sample by assay replicates
    dup<-f[rep,9]                     # group replicates
    
    if(length(which(is.na(dup)))==1){             
      singletons<-rbind(singletons,f[i,c(1,2,5,9)]) # if there is a singleton, add it to list
    }
    if(substr(f[i,5],1,1)=="m"){g[i,6]<-mean(dup)}else{g[i,6]<-mean(dup,na.rm=TRUE)}                           # add mean to row
  }
  
  # remove extra rows
  if(length(which(duplicated(g[,2:6])))>0){h<-g[-which(duplicated(g[,2:6])),]}else{h<-g}

  ### LOD SCREENING ###
  overLOD<-NULL
  samp<-which(h[,3]=="Unknown")
  
  if(!is.null(LOD)){
    if(LOD=="assay_LOD"){           # this is including all the LOD info from CSAS table 3
      abbrev<-c("ae_hyd","ae_sal","aspv","c_b_cys","ce_sha","cr_sal","de_sal","fa_mar", "fl_psy","gy_sal",         # the next ten lines or so create a datset of LODs
                "ic_hof","ic_mul","ihnv","ipnv","isav7","isav8","ku_thy","lo_sal","my_arc","my_cer","my_ins",
                "na_sal","ne_per","nu_sal","omv","pa_kab","pa_min","pa_pse","pa_ther","pch_sal","pisck_sal",
                "pmcv","prv","pspv","re_sal","rlo","sav","sch","sp_des","sp_sal","te_bry","ven","ver","vhsv",
                "vi_ang","vi_sal","ye_ruc")
      LOD<-c(28.7,25.6,26.2,25.7,28.5,24.3,25.5,30.6,29.5,26.4,24.2,23.7,27.6,27.6,27,26.1,26.2,25.4,26.8,26.2,
             26.4,24.3,25.4,26.1,26.6,25.6,29.6,25.2,28.2,30.7,23.3,26.3,26.1,26.4,25.9,25.2,26.3,27.9,26.5,26.1,
             25,24.9,26.2,26.9,26.4,25.8,25.8)
      assay_LOD<-as.data.frame(cbind(abbrev,LOD))
      assay_LOD[,2]<-as.numeric(as.character(assay_LOD[,2]))
      
      for(i in samp){                                       #here the assay specific LODs are applied
        if(substr(h[i,5],1,1)=="m" & !is.na(h[i,6])){
          abb<-substr(h[i,5],2,nchar(h[i,5]))
          lod<-assay_LOD[match(abb,assay_LOD[,1]),2]
          if(h[i,6]>lod)
          {overLOD<-rbind(overLOD,h[i,c(1,2,5,6)])
          h[i,6]<-NA}}}}
    else{                                                   #here the global LOD is applied
    for(i in samp){
      if(!is.na(h[i,6])){
        if(substr(h[i,5],1,1)=="m" & h[i,6]>LOD)
        {overLOD<-rbind(overLOD,h[i,c(1,2,5,6)])
         h[i,6]<-NA
        }}}
      assay_LOD<-NULL
      }
  }else{assay_LOD<-NULL}
  
  
  ### CHECK TO SEE IF THERE ARE ANY POSITIVE BLANKS ###
  b<-which(grepl("Blank",h$Stype))              # get the row numbers for all the blanks
  
  blankpos<-NULL                                # create object to hold data
  
  for(i in b){
    if(!is.na(h[i,6])){blankpos<-rbind(blankpos,h[i,c(1,2,5,6)])} #collect ID, Sample, Assay, and Fam Ct for positive blanks
  }
  
  
  #### TRANSPOSE DATA ####
  k<-data.frame(matrix(NA,nrow=length(unique(h$Sample)),ncol=length(unique(h$FAssay))+2)) # create new dataframe
  colnames(k)<-c("Sample",trim.leading(h[1:length(unique(h$FAssay)),5]),"RNAconc")                # place the Assay names as column names
  k[,1]<-h[seq(1,nrow(h),length(unique(h$FAssay))),2]
  
  count<-seq(1,nrow(h),length(unique(h$FAssay)))
   
  for(i in 1:length(unique(h$Sample))){
    k[i,2:(length(unique(h$FAssay))+1)]<-h[count[i]:(count[i]+length(unique(h$FAssay))-1),6]
    k[i,ncol(k)]<-h[count[i],4]
  }
  
  
  if(is.null(fails)){print("No assay - sample combinations were failed",quote=FALSE)}else{
    print(c(nrow(fails), "FAM positive assay - sample combinations were failed. To see the fails, enter (object name)$fails"),quote=FALSE)
  }  
  if(is.null(vicpos)){print("No FAM positive samples were found to be VIC positive",quote=FALSE)}else{
    print(c(nrow(vicpos), "samples were VIC positive. To see the samples, enter (object name)$vicpos. FAM Ct was replaced with NA if FAM was positive."),quote=FALSE)
  }
  if(is.null(singletons)){print("There were no singletons",quote=FALSE)}else{
    print(c(nrow(singletons), "samples were not detected in duplicate. For microbe assays, they are considered non-positive and removed. For biomarkers, the single value is adopted. To see the singletons, enter (object name)$single"),quote=FALSE)}
  if(is.null(LOD)){print("You elected to not perform Limit of Detection screening.",quote=FALSE)}else{
    if(is.null(overLOD)){print("No samples had Ct values beyond the Limit of Detection.",quote=FALSE)}else{
      print(c(nrow(overLOD), "samples had Ct values beyond the Limit of Detection. To see the samples, enter (object name)$overLOD. To see the LODs that were applied if you used assay_LOD, enter (object name)$LODlist"),quote=FALSE)
      
    }
  }
  
  if(is.null(blankpos)){print("No control blanks tested positive. Good job being clean!",quote=FALSE)}else{
    print(c(nrow(blankpos), "blanks were positive. You may have some contamination issues. To see which control blanks were positive, enter (object name)$blankpos"),quote=FALSE)
  }
  
  list(data = k, fails = fails, vicpos = vicpos, single = singletons, blankpos = blankpos, effdata = h, overLOD =overLOD,LODlist=assay_LOD)
  
  
}
