#' Calculate assay efficiencies and identify failed assays
#'@details This function will: 1) calculate efficiencies for each assay, 2) report the assays that failed (had less than 3 points in their standard curves), 3) return an object that contains the efficiency, N, y-intercept, and R-squared for each efficiency calculation (for use in later functions), 4) plot each standard curve if specified
#'@param effdata    this is an object created by the cleanavg() function
#'@param plot       If set to TRUE, standard curve plots for each assay will be produced. Default is NULL. At this time, the function can only plot for up to 60 assays at a time.
#'@return The function returns a dataframe containing efficiency (0-2), the number of points in the standard curve (non NAs), the r-squared, and the y-int. This object is applied to other functions in this package
#'@export

effcalc<-function(effdata,plot=NULL){
  
  m<-subset(effdata,effdata$Stype=="Standard")  # subset to only samples classified as standards
  
  assays<-unique(m[,5])
  eff<-data.frame(matrix(nrow=length(assays),ncol=6))
  colnames(eff)<-c("Assay","n","Slope","Y-int","E","R2")
  
  
  
  for(i in assays){
    s<-subset(m,m[,5]==i)
    if(substr(i,1,1)=="m"){t<-s[which(grepl("contclon",s[,2])),]}else{t<-s[which(grepl("contser",s[,2])),]}
    if(length(which(!is.na(t[,6])))>2)
    { fit<-lm(t[,6]~log10(t[,4]))
      eff[match(i,assays),2:6]<-c(length(which(!is.na(t[,6]))),as.numeric(round(fit$coef[2],2)),as.numeric(round(fit$coef[1],2)),as.numeric(round(10^(-1/fit$coef[2]),2)),round(summary(fit)$r.squared,3))
      if(!is.null(plot)){
        if(match(i,assays) %in% seq(1,95,2)){
        if(Sys.info()[1]=="Windows"){windows(10,5)}else{if(Sys.info()[1]=="Darwin"){quartz(10,5)}else{X11(10,5)}}
          par(mfrow=c(1,2))}     #open a double wide window to plot in, command depends upon                                      #      operating system. window only opens for every other
        plot(log10(t[,4]),t[,6],main=i,ylim=c(0,35))
        abline(fit)
        mtext(paste0("Slope=",as.numeric(round(fit$coef[2],2))),side=1,line=-1) 
        mtext(paste0("R2=",round(summary(fit)$r.squared,2)),side=1,line=-2)
        mtext(paste0("E=",round(10^(-1/fit$coef[2]),2)),side=1,line=-3)
      }}
  }
  fail<-which(is.na(eff$n))
  
  trim.leading <- function (x)  sub("^\\s+", "", x)               # introduce a function to remove blanks from assay names
  eff[,1]<-trim.leading(assays)
  if(length(fail)<1){print("No assays failed. Nice pippetting, gene jockey!",quote=FALSE)}else{print(c("The following assays had less than 3 positives, efficiency can not be calculated:",eff[fail,1]),quote=FALSE)}
  
  return(eff)
}
