#' Normalizes gene expression Ct values
#' @param chip  the data output from the cleanavg() function, or an object that is a combination of multiple chips.
#' @param effic the table output from the effcalc() function. Only needed when an efficiency correction (Pfaffl) method is used.
#' @param type  determines whether data is from one or more chips. Either 'single' (default) or 'multiple.grp_avg'. See more below in Details.
#' @param ref pick reference genes: either use "find" and code will find them (assay names beginning with 'r') or specify column numbers to use a subset of ref genes.
#' @param method options are 'Pfaffl_fc', 'Pfaffl_add', 'ddCt_fc', 'ddCt_add', 'dCt', or 'copy_no'. Explained below 
#' @param control  for single chips, the name of a reference sample within the chip (most likely a pooled sample). For multiple chips, a vector representing row numbers.
#' @details This function normalizes the biomarkers with several options for approach. While it carries the microbe data through to the end product, it does not alter microbe data. 
#' Control is by default one particular sample on the chip (for example, a pooled positive). Using an efficiency corrected method is recommended since the ddCt methods assume that efficiency is 100\%. Efficiency correction can be used when multiple chips are combined but note that it may not be appropriate since efficiencies vary between chips. Using efficiency averaged across chips is a potential solution but then might also defeat the purpose of efficiency correction in the first place.
#' 
#' 'Pfaffl_fc', and 'Pfaffl_add' use efficiency correction. 'fc' stands for fold change, the results will be expressed as the fold change difference from the control. 'add' means that you will have 
#' positive or negative numbers, with the reference gene as zero. The 'copy_no' converts both target and reference genes to RNA copy number using the standard curve generated in the efficiency() function.
#' This is an efficiency correction approach (only available for use with single chips). 'ddCt' methods are very simliar to Pfaffl, except that efficiency is assumed to be 100\%. 'dCt' is simply the difference between an individual's target and reference genes, no control included. It is generally not recommended. 
#' 
#' The default for the 'ref=' argument allows the code to locate your reference genes and create a geomean for each sample. If you want to use a subset of reference genes, simply specify the column numbers here (e.g., ref=c(45,46))
#' 
#' Note that the user has flexibility in defining control groups for ddCt and Pfaffl calculations. Defining one entire treatment group as a control will allow for quick estimations of fold
#' change associated with treatment (using ddCt_fc). Control groups can be user defined on single chips by setting type='multiple' (even though it is just one chip). If you are using a single control, type in the row name, if multiple, a vector of row numbers.
#' @export
 
normalize<-function(chip,effic,type="single",ref="find",method="ddCt_add",control="contpoolpos"){
  
  rcol<-NULL                             # create some empty objects
  bcol<-NULL
  mcol<-NULL
  misc<-NULL
  samps<-NULL
  
  
    
  for(i in 1:nrow(chip)){
    if(substr(chip[i,1],1,4)!="cont"){samps<-append(samps,i)}                # identifies any rows that are not controls and collects them
  }
  
  for(i in 1:ncol(chip)){                                                     # this just figures out what the columns are (as in above functions)
    letter<-substr(colnames(chip)[i],1,1)
    if(letter=="b"){bcol<-append(bcol,i)}
    else{if(letter=="m"){mcol<-append(mcol,i)}
      else{if(letter=="r" & ref[1]=="find"){rcol<-append(rcol,i)}
        else{if(letter=="r" & ref[1]!="find"){rcol<-ref}
          else{misc<-append(misc,i)}}}}}  
  
  if(type=="single"){
  
  if(method=="Pfaffl_fc"){                               # this is what happens if you use "Pfaffl_fc"
    
    c<-match(control,chip[,1])                        # find the control
    
    for (j in bcol){                                  # goes through one biomarker at a time
      for (i in samps){                                # for each non-control row
        ct.targ<-chip[c,j]-chip[i,j]                                                  # this caclulates Ct Target
        ct.ref<-exp(rowMeans(log(chip[c,rcol]),na.rm=TRUE))-exp(rowMeans(log(chip[i,rcol]),na.rm=TRUE))     # this caclulates Ct of the reference genes. Note that a geomean is used
        ern<-match(colnames(chip)[j],effic[,1])
        rern<-match(colnames(chip)[rcol],effic[,1])
        chip[i,j]<-(effic[ern,5]^ct.targ)/(mean(effic[rern,5])^ct.ref)           # this calculates R. Note that the reference gene efficiencies are averaged (arithmetic mean)
      }}}                                                                              # also note that result is placed directly back on chip
  
    if(method=="Pfaffl_add"){                               # this is what happens if you use "Pfaffl_add"
      
      c<-match(control,chip[,1])                        # find the control
      
      for (j in bcol){                                  # goes through one biomarker at a time
        for (i in samps){                                # for each non-control row
          ct.targ<-chip[c,j]-chip[i,j]                                                  # this caclulates Ct Target
          ct.ref<-exp(rowMeans(log(chip[c,rcol]),na.rm=TRUE))-exp(rowMeans(log(chip[i,rcol]),na.rm=TRUE))        # this caclulates Ct of the reference genes. Note that a geomean is used
          ern<-match(colnames(chip)[j],effic[,1])
          rern<-match(colnames(chip)[rcol],effic[,1])
          chip[i,j]<-log2((effic[ern,5]^ct.targ)/(mean(effic[rern,5])^ct.ref))           # this calculates R. Note that the reference gene efficiencies are averaged (arithmetic mean)
        }}}                                                                              # also note that result is placed directly back on chip
   
#     if(method=="Pfaffl_dCt"){                               # this is what happens if you use "Pfaffl_dCt"
      
#      c<-match(control,chip[,1])                        # find the control
      
#      for (j in bcol){                                  # goes through one biomarker at a time
#        for (i in samps){                                # for each non-control row
#          ct.targ<-chip[c,j]-chip[i,j]                                                  # this caclulates Ct Target
#          ern<-match(colnames(chip)[j],effic[,1])
#          chip[i,j]<-effic[ern,5]^ct.targ          
#        }}}                                                                              
    
    if(method=="ddCt_fc"){                               # this is what happens if you use "ddCt_fc"
      
      c<-match(control,chip[,1])                        # find the control
      
      for (j in bcol){
        for (i in samps){
          ct.cont<-chip[c,j]-exp(rowMeans(log(chip[c,rcol]),na.rm=TRUE))     # difference between the "control" target and reference genes
          ct.trt<-chip[i,j]-exp(rowMeans(log(chip[i,rcol]),na.rm=TRUE))       # difference between the "treated" target and reference genes
          chip[i,j]<-2^(ct.cont-ct.trt)                             # calculate delta delta Ct as fold change and place on chip
        }}
    }
  
  if(method=="ddCt_add"){                               # this is what happens if you use "ddCt_add"
    
    c<-match(control,chip[,1])                        # find the control
    
    for (j in bcol){
      for (i in samps){
        ct.cont<-chip[c,j]-exp(rowMeans(log(chip[c,rcol]),na.rm=TRUE))     # difference between the "control" target and reference genes
        ct.trt<-chip[i,j]-exp(rowMeans(log(chip[i,rcol])))        # difference between the "treated" target and reference genes
        chip[i,j]<-ct.cont-ct.trt                             # calculate delta delta Ct and place on chip
      }}
  }
  
  if(method=="dCt"){
    for (j in bcol){
      for (i in samps){
        ct.trt<-exp(rowMeans(log(chip[i,rcol]),na.rm=TRUE))-chip[i,j]        # difference between the  target and reference genes
        chip[i,j]<-ct.trt                           # calculate delta delta Ct and place on chip
      }}
  }
    
  if(method=="copy_no_ratio"){
    for (j in bcol){
      erow<-match(colnames(chip)[j],effic[,1])
      for(i in samps){
        abs.targ<-10^((chip[i,j]-effic[erow,4])/effic[erow,3])
        rrows<-match(colnames(chip)[rcol],effic[,1])
       rabs2<-NULL
        for(x in rcol){
          rabs<-10^((chip[i,x]-effic[rrows[match(x,rcol)],4])/effic[rrows[match(x,rcol)],3])
          rabs2<-append(rabs2,rabs)
          }
        abs.ref<-exp(mean(log(rabs2)))  
        chip[i,j]<-abs.targ/abs.ref
        }
      }
    }
  }  
  
  
  if(type=="multiple.grp_avg"){
  #  control is now a vector of numbers indicating row, however it can still be a single sample!  
  chip.x<-chip # protecting the chip data so that everything can be used as a control
      
    if(method=="Pfaffl_fc"){                              
      for (j in bcol){
        for (i in samps){
          ct.targ<-mean(chip[control,j],na.rm=TRUE)-chip[i,j] 
          ct.ref<-mean(exp(rowMeans(log(chip[control,rcol]),na.rm=TRUE)),na.rm=TRUE)-exp(rowMeans(log(chip[i,rcol]),na.rm=TRUE))     # this caclulates Ct of the reference genes. Note that a geomean is used
          ern<-match(colnames(chip)[j],effic[,1])
          rern<-match(colnames(chip)[rcol],effic[,1])
          chip.x[i,j]<-(effic[ern,5]^ct.targ)/(mean(effic[rern,5])^ct.ref)   
        }}
      chip<-chip.x
      }
    
    if(method=="Pfaffl_add"){                               
      for (j in bcol){
        for (i in samps){
          ct.targ<-mean(chip[control,j],na.rm=TRUE)-chip[i,j] 
          ct.ref<-mean(exp(rowMeans(log(chip[control,rcol]),na.rm=TRUE)),na.rm=TRUE)-exp(rowMeans(log(chip[i,rcol]),na.rm=TRUE))     # this caclulates Ct of the reference genes. Note that a geomean is used
          ern<-match(colnames(chip)[j],effic[,1])
          rern<-match(colnames(chip)[rcol],effic[,1])
          chip.x[i,j]<-log2((effic[ern,5]^ct.targ)/(mean(effic[rern,5])^ct.ref))  
        }}
      chip<-chip.x
      }
    
        
    if(method=="ddCt_fc"){                               # this is what happens if you use "ddCt_fc"
      
      
      for (j in bcol){
        for (i in samps){
          ct.cont<-mean(chip[control,j],na.rm=TRUE)-mean(exp(rowMeans(log(chip[control,rcol]),na.rm=TRUE)),na.rm=TRUE)     # difference between the "control" target and reference genes
          ct.trt<-chip[i,j]-exp(rowMeans(log(chip[i,rcol]),na.rm=TRUE))        # difference between the "treated" target and reference genes
          chip.x[i,j]<-2^(ct.cont-ct.trt)                             # calculate delta delta Ct as fold change and place on chip
        }}
      chip<-chip.x
    }
    
    if(method=="ddCt_add"){                               # this is what happens if you use "ddCt_add"
      

      for (j in bcol){
        for (i in samps){
          ct.cont<-mean(chip[control,j],na.rm=TRUE)-mean(exp(rowMeans(log(chip[control,rcol]),na.rm=TRUE)),na.rm=TRUE)     # difference between the "control" target and reference genes
          ct.trt<-chip[i,j]-exp(rowMeans(log(chip[i,rcol])))        # difference between the "treated" target and reference genes
          chip.x[i,j]<-ct.cont-ct.trt                             # calculate delta delta Ct and place on chip
        }}
      chip<-chip.x
    }
    
    if(method=="dCt"){
      for (j in bcol){
        for (i in samps){
          ct.trt<-exp(rowMeans(log(chip[i,rcol]),na.rm=TRUE))-chip[i,j]        # difference between the "treated" target and reference genes
          chip.x[i,j]<-ct.trt                           # calculate delta delta Ct and place on chip
        }}
      chip<-chip.x
    }
  }
  
  return(chip)        # note that reference genes, all controls, and microbes will still be expressed in Ct
}
